@@ -1,8 +1,9 @@
 // jslint configuration
 /*jslint browser: true */
-/*global $, jQuery, OpenLayers, window, map, updateLayer, fillSidebar,
+/*global $, jQuery, OpenLayers, window, map, fillSidebar,
 setUpScreen, nothingFoundPopup, reloadGraphs, reloadMapActions,
-setUpTransparencySlider, setUpAnimationSlider, setUpTooltips */
+setUpTransparencySlider, setUpAnimationSlider, setUpTooltips,
+refreshLayers */
 
 
 /*
@@ -55,6 +56,10 @@ jQuery.fn.liveCheckboxes = function () {
 /* Shows an "jquery tools overlay" popup, data must have the following properties:
 data.html
 
+Uses customized "default" effect to make tabs work correctly. See
+bottom of this file. It works, but it loads graphs 2 times in case of
+tabs. Needs fixing.
+
 Previously it also needed:
 data.id
 data.x
@@ -63,11 +68,31 @@ data.big
 But those aren't used anymore.
 */
 
-function show_popup(data, map) {
+function show_popup(data) {
+    var html, overlay, i;
     if (data !== null) {
-        if (data.html !== "" && data.html !== undefined) {
-            var overlay;
-            $('#graph-popup-content').html(data.html);
+        if (data.html && data.html.length !== 0) {
+            // Generates pages with handlers. First only page 0 is visible.
+            if (data.html.length === 1) {
+                $("#graph-popup-content").html(data.html[0]);
+            } else {
+                // Build up html with tabs.
+                html = "<ul class=\"tabs css-tabs\">";
+                for (i = 0; i < data.html.length; i += 1) {
+                    html += "<li><a href=\"tab-" + (i + 1) + "\">Resultaat ";
+                    html += (i + 1) + "</a></li>";
+                }
+                html += "</ul>";
+                for (i = 0; i < data.html.length; i += 1) {
+                    html += "<div class=\"pane\"><div class=\"pane-content\">";
+                    html += data.html[i];
+                    html += "</div></div>";
+                }
+
+                $("#graph-popup-content").html(html);
+                $(".tabs").tabs("div.pane > div.pane-content");
+                // #graph-popup-content
+            }
             overlay = $('#graph-popup').overlay();
             overlay.load();
             reloadGraphs();
@@ -89,26 +114,25 @@ function hover_popup(data, map) {
         $("#hover-popup").remove(); // remove existing popup, if exists
         popup = new OpenLayers.Popup('hover-popup',
                                      new OpenLayers.LonLat(data.x, data.y),
-                                     new OpenLayers.Size(300, 35),
+                                     new OpenLayers.Size(300, 300),
                                      content,
                                      false);
-        popup.maxSize = new OpenLayers.Size(300, 35);
+        popup.maxSize = new OpenLayers.Size(300, 300);
         popup.border = "1px solid black";
         popup.autoSize = true;
         map.addPopup(popup);
     }
 }
 
-
 jQuery.fn.collagePopup = function () {
     var url, collage_id;
-    url = $(this).parent().attr("data-url-lizard-map-collage-popup");
-    collage_id = $(this).attr("data-collage-id");
+    url = $(this).closest("[data-url-lizard-map-collage-popup]").attr("data-url-lizard-map-collage-popup");
+    collage_id = $(this).closest("[data-collage-id]").attr("data-collage-id");
     $.getJSON(
         url,
         { collage_id: collage_id },
         function (data) {
-            show_popup(data, map);
+            show_popup(data);
         }
     );
 };
@@ -181,17 +205,18 @@ jQuery.fn.workspaceInteraction = function () {
                      adapter_layer_json: adapter_layer_json
                     },
                     function (workspace_id) {
+                        // "click" the empty temp workspace. To be removed.
+                        $(".workspace-empty-temp").click();
                         // very strange... $workspace becomes the
                         // <ul> element, using workspaceItems...
                         // TODO: empty temp workspace
                         workspaceItems.parent().parent().updateWorkspace();
-                        $(".workspace-empty-temp").click(); // "click" the empty temp workspace
                     }
                 );
             }
         });
-        // Make collage clickable. (TODO: should be collage-popup)
-        $(".collage", $workspace).live('click',
+        // Make collage clickable. (DONE: should be collage-popup)
+        $(".collage-popup", $workspace).live('click',
                                        $(this).collagePopup);
         // Snippets. Using sortable instead of draggable because
         // Draggable applies to li and sortable applies to ul element
@@ -209,7 +234,7 @@ jQuery.fn.workspaceInteraction = function () {
                 url,
                 { snippet_id: snippet_id },
                 function (data) {
-                    show_popup(data, map);
+                    show_popup(data);
                 }
             );
             //snippet(snippet_id, map); // attention: from krw_waternet.js
@@ -231,8 +256,6 @@ jQuery.fn.updateWorkspace = function () {
         var $workspace, workspace_id, $holder;
         $workspace = $(this);
         workspace_id = $workspace.attr("data-workspace-id");
-        // reload map layers
-        updateLayer(workspace_id); // from lizardgis
         // reload workspace items: TODO: works only with a single workspace
         // item.
         $holder = $('<div/>');
@@ -243,9 +266,16 @@ jQuery.fn.updateWorkspace = function () {
             function () {
                 $(".workspace-items", $workspace).html(
                     $('.workspace-items', $holder).html());
+                $(".snippet-list", $workspace).html(
+                    $('.snippet-list', $holder).html());
                 fillSidebar();
                 $(".map-actions").html(
                     $('.map-actions', $holder).html());
+                $("#lizard-map-wms").html(
+                    $('#lizard-map-wms', $holder).html());
+                // reload map layers
+                refreshLayers(); // from lizard_wms.js
+                // Is this enough? See also refreshMapActionsDivs in lizard_map
                 setUpAnimationSlider();
                 setUpTransparencySlider();
                 setUpTooltips();
@@ -308,6 +338,12 @@ function workspaceItemOrSnippet(object) {
     //.workspace_item .snippet
 }
 
+
+function addProgressAnimationIntoWorkspace() {
+    $("#trash1").after('<img src="/static_media/lizard_ui/ajax-loader3.gif" class="sidebarbox-action-progress" data-src="" />');
+}
+
+
 /* Make a workspace trashbox
 
 accepts workspace_item classes
@@ -336,6 +372,7 @@ jQuery.fn.workspaceTrashBox = function () {
             activeClass: 'dropactive',
             drop: function (event, ui) {
                 var object_id, url;
+                addProgressAnimationIntoWorkspace();
                 object_id = ui.draggable.attr("data-object-id");
                 ui.draggable.remove();  // for visual snappyness
                 if (ui.draggable.is(".workspace-item")) {
@@ -391,3 +428,14 @@ jQuery.fn.lizardMapLink = function () {
         }
     });
 };
+
+
+$(document).ready(function () {
+    // Change "default" effect: reload graphs to fix layout.
+    $.tools.tabs.addEffect("default", function(tabIndex, done) {
+        // hide all panes and show the one that is clicked
+	this.getPanes().hide().eq(tabIndex).show();
+        reloadGraphs();
+        done.call();
+    });
+});
