@@ -1,8 +1,8 @@
 // jslint configuration; btw: don't put a space before 'jslint' below.
 /*jslint browser: true */
-/*global $, OpenLayers, window, updateLayer, updateLayers,
+/*global $, OpenLayers, window, updateLayers, addProgressAnimationIntoWorkspace,
 stretchOneSidebarBox, reloadGraphs, fillSidebar, show_popup,
-hover_popup, layers, map */
+hover_popup, layers, map, refreshLayers */
 
 var animationTimer, transparencyTimer;
 
@@ -85,92 +85,61 @@ function setUpMapLoadDefaultLocation() {
         url = $(this).attr("data-url");
         $.getJSON(
             url, function (data) {
-                if ((data.x !== undefined) &&
-                    (data.y !== undefined) &&
-                    (data.zoom !== undefined))
-                {
-                    map.setCenter(
-                        new OpenLayers.LonLat(parseFloat(data.x),
-                                              parseFloat(data.y)),
-                        parseFloat(data.zoom));
+                var extent;
+                if (data.extent !== undefined) {
+                    extent = new OpenLayers.Bounds(
+                        data.extent.left,
+                        data.extent.top,
+                        data.extent.right,
+                        data.extent.bottom);
+                    map.setCenter(extent.getCenterLonLat(),
+                                  map.getZoomForExtent(extent) - 1);
                 }
             });
     });
 }
 
 
-/* Reload map-actions: put all initialization functions here for map actions
-(above map, load/save location/ empty temp workspace) */
-function reloadMapActions() {
-    $(".map-actions").load(
-        "./ .map-action",
-        function () {
-            fillSidebar();
-            setUpAnimationSlider();
-            setUpTransparencySlider();
-            setUpMapLoadDefaultLocation();
-        });
+/* set up map actions */
+function setUpMapActions() {
+    fillSidebar();
+    setUpAnimationSlider();
+    setUpTransparencySlider();
+    setUpMapLoadDefaultLocation();
 }
 
 
-function setUpWorkspaceAcceptable() {
-    // Set up draggability for current and future items.
-    // See http://tinyurl.com/29lg4y3 .
-    $(".workspace-acceptable").live("mouseover", function () {
-        if (!$(this).data("draggable-initialized")) {
-            $(this).data("draggable-initialized", true);
-            $(this).draggable({
-                scroll: 'false',
-                cursor: 'move',
-                helper: 'clone',
-                appendTo: 'body',
-                revert: 'true',
-                placeholder: 'ui-sortable-placeholder'
-            });
-        }
-    });
-    // Clicking a workspace-acceptable shows it in the 'temp' workspace.
-    $(".workspace-acceptable").live("click", function (event) {
-        var name, adapter_class, adapter_layer_json, url_add_item_temp;
-        $(".workspace-acceptable").removeClass("selected");
-        $(this).addClass("selected");
-        name = $(this).attr("data-name");
-        adapter_class = $(this).attr("data-adapter-class");
-        adapter_layer_json = $(this).attr("data-adapter-layer-json");
-        url_add_item_temp = $(".workspace").attr(
-            "data-url-lizard-map-session-workspace-add-item-temp");
-        $.post(
-            url_add_item_temp,
-            {name: name,
-             adapter_class: adapter_class,
-             adapter_layer_json: adapter_layer_json
-            },
-            function (workspace_id) {
-                var url, center_x, center_y;
-                updateLayer(workspace_id);
-                reloadMapActions();
-                // Load extent from workspace and zoom to it.
-                url = $(".workspace").attr(
-                    "data-url-lizard-map-session-workspace-extent-temp");
-                $.getJSON(url, {}, function (extent) {
-                    // If we do not get extent, just forget it.
-                    if ((extent.north !== null) &&
-                        (extent.south !== null) &&
-                        (extent.east !== null) &&
-                        (extent.west !== null))
-                    {
-                        // Convert bbox to center coordinates
-                        center_x = (extent.east + extent.west) / 2;
-                        center_y = (extent.north + extent.south) / 2;
-                        // Now pan
-                        map.panTo(
-                            new OpenLayers.LonLat(parseFloat(center_x),
-                                                  parseFloat(center_y)));
-                    }
-                });
-            });
-        stretchOneSidebarBox();
-    });
+/* Refreshes map-actions and lizard-map-wms divs, then inits js. */
+/* Looks a bit like jQuery.fn.updateWorkspace part. Needs refactoring. */
+function refreshMapActionsDivs() {
+    var $holder;
+    $holder = $('<div/>');
+    $holder.load(
+        './ #page',
+        function () {
+            $(".map-actions").html(
+                $('.map-actions', $holder).html());
+            $("#lizard-map-wms").html(
+                $('#lizard-map-wms', $holder).html());
+            refreshLayers();
+            setUpMapActions();
+        });
+}
+
+/* Reload map-actions: put all initialization functions here for map actions
+(above map, load/save location/ empty temp workspace)
+
+OBSOLETE, should be integrated with other load functions
+*/
+function reloadMapActions() {
+    $(".map-actions").load(
+        "./ .map-action",
+        setUpMapActions
+    );
+}
+
+
+function setUpAddWorkspaceItem() {
 
     // Add action to button 'add-workspace-item' in workspace
     // acceptables. This class is added by
@@ -209,52 +178,134 @@ function setUpWorkspaceAcceptable() {
 }
 
 
+function setUpWorkspaceAcceptable() {
+    // Set up draggability for current and future items.
+    // See http://tinyurl.com/29lg4y3 .
+    $(".workspace-acceptable").live("mouseover", function () {
+        var html;
+        if (!$(this).data("draggable-initialized")) {
+            $(this).data("draggable-initialized", true);
+            $(this).draggable({
+                scroll: 'false',
+                cursor: 'move',
+                helper: 'clone',
+                appendTo: 'body',
+
+                revert: 'true',
+                placeholder: 'ui-sortable-placeholder'
+            });
+        }
+        if (!$(this).data("add-workspace-item-initialized")) {
+            // Add the "add to workspace" button
+            $(this).data("add-workspace-item-initialized", true);
+            html = $(this).html();
+            html = html + '<span class="ss_sprite ss_add sidebarbox-action-icon add-workspace-item" title="Voeg laag toe aan workspace">&nbsp;</span>';
+            $(this).html(html);
+        }
+    });
+    // Do not use mouseout: Mouseout is also activated when moving to
+    // the "add-workspace-item" button.
+    $(".workspace-acceptable").live("mouseleave", function () {
+        //  Remove all "add to workspace" button(s).
+        $(".add-workspace-item").remove();
+        $(".workspace-acceptable").removeData(
+            "add-workspace-item-initialized");
+    });
+
+    // Clicking a workspace-acceptable shows it in the 'temp' workspace.
+    $(".workspace-acceptable").live("click", function (event) {
+        var name, adapter_class, adapter_layer_json, url_add_item_temp, $workspace, html;
+        $(".workspace-acceptable").removeClass("selected");
+        $(this).addClass("selected");
+        name = $(this).attr("data-name");
+        adapter_class = $(this).attr("data-adapter-class");
+        adapter_layer_json = $(this).attr("data-adapter-layer-json");
+        $workspace = $(".workspace");
+        url_add_item_temp = $workspace.attr(
+            "data-url-lizard-map-session-workspace-add-item-temp");
+        $.post(
+            url_add_item_temp,
+            {name: name,
+             adapter_class: adapter_class,
+             adapter_layer_json: adapter_layer_json
+            },
+            function (workspace_id) {
+                var url, center_x, center_y;
+                refreshMapActionsDivs();
+                // Load extent from workspace and zoom to it.
+                url = $(".workspace").attr(
+                    "data-url-lizard-map-session-workspace-extent-temp");
+                $.getJSON(url, {}, function (extent) {
+                    var ol_extent;
+                    // If we do not get extent, just forget it.
+                    if ((extent.north !== null) &&
+                        (extent.south !== null) &&
+                        (extent.east !== null) &&
+                        (extent.west !== null))
+                    {
+                        ol_extent = new OpenLayers.Bounds(
+                            extent.west, extent.north,
+                            extent.east, extent.south);
+                        map.setCenter(ol_extent.getCenterLonLat(),
+                                      map.getZoomForExtent(ol_extent) - 3);
+                    }
+                });
+            });
+        stretchOneSidebarBox();
+        return false;
+    });
+}
+
+
 function setUpWorkspaceEmpty() {
     var $workspace, workspace_id, url;
     $(".workspace-empty-trigger").live('click', function () {
         $workspace = $(this).parents("div.workspace");
         workspace_id = $workspace.attr("data-workspace-id");
         url = $workspace.attr("data-url-lizard-map-workspace-item-empty");
+        addProgressAnimationIntoWorkspace();
         $.post(
             url, {workspace_id: workspace_id},
-            function (data) {
+	    function (data) {
+	        //remove progress
+                $workspace.find(".sidebarbox-action-progress").remove();
                 $workspace.updateWorkspace();
             });
     });
 }
 
 
-// Date selector: only support for ONE date selector at the moment.
-
-function setUpDateChoice() {
-    $.datepicker.setDefaults($.datepicker.regional.nl);
-    $("#id_date_start").datepicker();
-    $("#id_date_end").datepicker();
+/* Updates the date popup from a select or input tag */
+function updateDateSelectOrInput() {
+    var url, $form;
+    $form = $(this).parents("form");
+    url = $form.attr("action");
+    $.post(
+        url,
+        $form.serialize(),
+        function () {
+            // Update the popup. Note: We cannot use load, because the
+            // overlay properties will get lost
+            $.get("./", {}, function (data) {
+                var new_contents, curr_period_title;
+                new_contents = $(data).find(
+                    "#summary-datepicker-contents").html();
+                $("#summary-datepicker-contents").html(new_contents);
+                curr_period_title = $(data).find(
+                    "#summary-datepicker-a").attr("title");
+                $("#summary-datepicker-a").attr("title", curr_period_title);
+            });
+            reloadGraphs();
+        });
 }
 
-function setUpDateAjaxForm(overlay) {
-    var form = $("form", overlay);
-    form.submit(function () {
-        $.post(
-            form.attr('action'),
-            form.serialize(),
-            function (data) {
-                // This is the success handler.  Form can contain errors,
-                // though.
-                var newForm, freshForm;
-                newForm = $(data).find('form');
-                form.html(newForm);
-                setUpDateChoice();
-                freshForm = $("form", overlay);
-                setUpDateAjaxForm(freshForm);
-                if (newForm.html().indexOf("rror") === -1) {
-                    // No error/Error, so refresh graphs and close the popup.
-                    reloadGraphs();
-                    $("div.close", overlay).click();
-                }
-            });
-        return false;
-    });
+
+// Updates date div from server when fields change.
+function setUpDateUpdate() {
+    $("#summary-datepicker form input").live(
+        'change', updateDateSelectOrInput);
+    $("#summary-datepicker form select").live(
+        'change', updateDateSelectOrInput);
 }
 
 
@@ -262,37 +313,31 @@ function setUpDatePopup() {
     $(".popup-trigger").live('mouseover', function () {
         if (!$(this).data("popup-initialized")) {
             $(this).data("popup-initialized", true);
-            $(this).overlay({
-                onLoad: function () {
-                    var overlay = this.getOverlay();
-                    setUpDateAjaxForm(overlay);
-                }
-            });
+            $(this).overlay();
         }
     });
 }
 
+
 function setUpNotFoundPopup() {
     $("#not_found_popup_trigger").overlay();
 }
 
+
 function nothingFoundPopup() {
     $("#not_found_popup_trigger").click();
     setTimeout(function () {
-        $("#not_found_popup div.close").click();
+        $("#not_found_popup .close").click();
     },
               2000);
 }
 
+
 function setUpGraphEditPopup() {
     $(".graph_edit_trigger").overlay();
 }
 
 
-
-
-
-
 /*
 Empty the temp workspace
 */
@@ -307,9 +352,7 @@ function setUpEmptyTempInteraction() {
             url,
             {object_id: workspace_item_id},
             function (workspace_id) {
-                updateLayer(workspace_id);
-                // load new map actions
-                reloadMapActions();
+                refreshMapActionsDivs();
                 // remove highlighting
                 $(".workspace-acceptable").removeClass("selected");
             }
@@ -318,35 +361,43 @@ function setUpEmptyTempInteraction() {
 }
 
 
+/* Handle a click */
+/* Assumes there is 1 "main" workspace. Adds workspace_id to request. Only required when viewing workspaces of others */
 function popup_click_handler(x, y, map) {
-    var extent, radius, url;
+    var extent, radius, url, user_workspace_id;
     extent = map.getExtent();
     radius = Math.abs(extent.top - extent.bottom) / 30;  // Experimental, seems to work good
     $("#map_OpenLayers_ViewPort").css("cursor", "progress");
     url = $(".workspace").attr("data-url-lizard-map-search-coordinates");
+    user_workspace_id = $(".workspace").attr("data-workspace-id");
     if (url !== undefined) {
         $.getJSON(
             url,
-            { x: x, y: y, radius: radius, srs: map.getProjection() },
+            { x: x, y: y, radius: radius, srs: map.getProjection(),
+              user_workspace_id: user_workspace_id},
             function (data) {
                 $("#map_OpenLayers_ViewPort").css("cursor", "default");
-                show_popup(data, map);
+                show_popup(data);
             }
         );
     }
 }
 
 
+/* Handle a hover */
+/* Assumes there is 1 "main" workspace. Adds workspace_id to request. Only required when viewing workspaces of others */
 function popup_hover_handler(x, y, map) {
     /* Show name of one item when hovering above a map */
-    var extent, radius, url;
+    var extent, radius, url, user_workspace_id;
     extent = map.getExtent();
     radius = Math.abs(extent.top - extent.bottom) / 30;  // experimental, seems to work good
     url = $(".workspace").attr("data-url-lizard-map-search-name");
+    user_workspace_id = $(".workspace").attr("data-workspace-id");
     if (url !== undefined) {
         $.getJSON(
             url,
-            { x: x, y: y, radius: radius, srs: map.getProjection() },
+            { x: x, y: y, radius: radius, srs: map.getProjection(),
+              user_workspace_id: user_workspace_id},
             function (data) {
                 hover_popup(data, map);
             }
@@ -415,20 +466,18 @@ function setUpWorkspaceItemPanToLayer() {
             url,
             {workspace_item_id: workspace_item_id},
             function (extent) {
-                var center_x, center_y;
+                var center_x, center_y, ol_extent;
                 // If we do not get extent, just forget it.
                 if ((extent.north !== null) &&
                     (extent.south !== null) &&
                     (extent.east !== null) &&
                     (extent.west !== null))
                 {
-                    // Convert bbox to center coordinates
-                    center_x = (extent.east + extent.west) / 2;
-                    center_y = (extent.north + extent.south) / 2;
-                    // Now pan
-                    map.panTo(
-                        new OpenLayers.LonLat(parseFloat(center_x),
-                                              parseFloat(center_y)));
+                    ol_extent = new OpenLayers.Bounds(
+                        extent.west, extent.north,
+                        extent.east, extent.south);
+                    map.setCenter(ol_extent.getCenterLonLat(),
+                                  map.getZoomForExtent(ol_extent) - 3);
                 }
             });
     });
@@ -464,13 +513,16 @@ function setUpLegendEdit() {
 
 
 function mapSaveLocation() {
-    var url, coordinates;
+    var url, extent;
     url = $("#lizard-map-wms").attr("data-save-location-url");
-    coordinates = map.center;
+    extent = map.getExtent();
     $.ajax({
         type: 'POST',
         url: url,
-        data: {x: coordinates.lon, y: coordinates.lat, zoom: map.zoom},
+        data: {bottom: extent.bottom,
+               left: extent.left,
+               right: extent.right,
+               top: extent.top},
         async: false,
         success: function () {}
     });
@@ -479,10 +531,11 @@ function mapSaveLocation() {
 
 // Initialize all workspace actions.
 $(document).ready(function () {
+    setUpAddWorkspaceItem();
     setUpWorkspaceAcceptable();
     setUpWorkspaceEmpty();
     setUpDatePopup();
-    setUpDateChoice();
+    setUpDateUpdate();
     setUpNotFoundPopup();
     setUpEmptyTempInteraction();
     setUpAnimationSlider();
@@ -497,7 +550,9 @@ $(document).ready(function () {
 
     /* Workspace functions, requires jquery.workspace.js */
     $(".workspace").workspaceInteraction();
-    $(".add-snippet").snippetInteraction(); // voor collage view, nu nog nutteloos voor popup
+
+    // voor collage view, nu nog nutteloos voor popup
+    $(".add-snippet").snippetInteraction();
     $("a.lizard-map-link").lizardMapLink();
 });
 
