@@ -26,6 +26,7 @@ GROUPING_HINT = 'grouping_hint'
 USER_WORKSPACES = 'user'
 DEFAULT_WORKSPACES = 'default'
 TEMP_WORKSPACES = 'temp'
+OTHER_WORKSPACES = 'other'  # for flexible adding workspaces of others
 
 # TODO: Can this property be moved to mapnik_helper?
 ICON_ORIGINALS = pkg_resources.resource_filename('lizard_map', 'icons')
@@ -34,6 +35,9 @@ ADAPTER_ENTRY_POINT = 'lizard_map.adapter_class'
 SEARCH_ENTRY_POINT = 'lizard_map.search_method'
 LOCATION_ENTRY_POINT = 'lizard_map.location_method'
 
+# WMS is a special kind of adapter: the client side behaves different.
+ADAPTER_CLASS_WMS = 'wms'
+
 logger = logging.getLogger(__name__)
 # Add introspection rules for ColorField
 add_introspection_rules([], ["lizard_map.models.ColorField"])
@@ -161,6 +165,16 @@ class AdapterClassNotFoundError(Exception):
     pass
 
 
+class WorkspaceItemError(Exception):
+    """To be raised when a WorkspaceItem is out of date.
+
+    A WorkspaceItem can represent something that does no longer exist.
+    For example, it may refer to a shape that has been deleted from
+    the database. This error may trigger deletion of such orphans.
+    """
+    pass
+
+
 ###### Models start here ######
 
 
@@ -205,6 +219,34 @@ class Workspace(models.Model):
                 north = wsi_extent['north']
         return {'north': north, 'south': south, 'east': east, 'west': west}
 
+    def wms_layers(self):
+        """
+        Returns a list of wms_layers. Each wms_layer is a dict with keys:
+        wms_id, name, url, params, options. They are used in wms.html
+        """
+        result = []
+        for workspace_item in self.workspace_items.filter(
+            adapter_class=ADAPTER_CLASS_WMS, visible=True):
+
+            # The special WMS layer arguments provides name, url,
+            # params, options.
+            layer_arguments = workspace_item.adapter_layer_arguments
+            layer_arguments.update(
+                {'wms_id': '%d_%d' % (self.id, workspace_item.id)})
+            result.append(layer_arguments)
+
+        result.reverse()
+        return result
+
+    @property
+    def is_animatable(self):
+        """Determine if any visible workspace_item is animatable."""
+        for workspace_item in self.workspace_items.filter(visible=True):
+            if workspace_item.adapter.is_animatable:
+                return True
+        return False
+
+
 class WorkspaceItem(models.Model):
     """Can show things on a map based on configuration in a url."""
 
@@ -237,12 +279,17 @@ class WorkspaceItem(models.Model):
             if entrypoint.name == self.adapter_class:
                 try:
                     real_adapter = entrypoint.load()
+                    real_adapter = real_adapter(self,
+                        layer_arguments=self.adapter_layer_arguments)
                 except ImportError, e:
                     logger.critical("Invalid entry point: %s", e)
                     raise
-                return real_adapter(
-                    self,
-                    layer_arguments=self.adapter_layer_arguments)
+                except WorkspaceItemError:
+                    logger.warning(
+                        "Deleting problematic WorkspaceItem: %s", self)
+                    # Trac #2470. Return a NullAdapter instead?
+                    self.delete()
+                return real_adapter
         raise AdapterClassNotFoundError(
             u'Entry point for %r not found' % self.adapter_class)
 
@@ -264,6 +311,16 @@ class WorkspaceItem(models.Model):
         """Can I provide a adapter class for i.e. WMS layer?"""
         return bool(self.adapter_class)
 
+    def has_extent(self):
+        """
+        Return true if workspace item has a extent that makes sense.
+        """
+        extent = self.adapter.extent()
+        if None in extent.values():
+            return False
+        else:
+            return True
+
     def delete(self, *args, **kwargs):
         """
         When deleting a WorkspaceItem, delete corresponding snippets
@@ -675,7 +732,9 @@ class Legend(models.Model):
 
     @property
     def float_format(self):
-        """Determines float format for defined legend. Required by legend_default."""
+        """Determine float format for defined legend.
+
+        Required by legend_default."""
         delta = abs(self.max_value - self.min_value) / self.steps
         if delta < 0.1:
             return '%.3f'
@@ -782,7 +841,7 @@ class Legend(models.Model):
     def mapnik_linestyle(self, value_field=None):
         """Deprecated. Use mapnik_style instead. Return a Mapnik
         line/polystyle from Legend object"""
-        return mapnik_style(self, value_field=value_field)
+        return self.mapnik_style(self, value_field=value_field)
 
 
 class LegendPoint(Legend):
@@ -818,7 +877,8 @@ class LegendPoint(Legend):
         mapnik_filter = "[%s] <= %f" % (value_field, self.min_value)
         logger.debug('adding mapnik_filter: %s' % mapnik_filter)
         mapnik_rule = point_rule(
-            self.icon, self.mask, self.too_low_color, mapnik_filter=mapnik_filter)
+            self.icon, self.mask, self.too_low_color,
+            mapnik_filter=mapnik_filter)
         mapnik_style.rules.append(mapnik_rule)
 
         # in boundaries
@@ -841,3 +901,93 @@ class LegendPoint(Legend):
         mapnik_style.rules.append(mapnik_rule)
 
         return mapnik_style
+
+
+class BackgroundMap(models.Model):
+    """
+    Defines background maps.
+    """
+    LAYER_TYPE_GOOGLE = 1
+    LAYER_TYPE_OSM = 2
+    LAYER_TYPE_WMS = 3
+
+    LAYER_TYPE_CHOICES = (
+        (LAYER_TYPE_GOOGLE, 'GOOGLE'),
+        (LAYER_TYPE_OSM, 'OSM'),
+        (LAYER_TYPE_WMS, 'WMS'),
+        )
+
+    GOOGLE_TYPE_DEFAULT = 1
+    GOOGLE_TYPE_PHYSICAL = 2
+    GOOGLE_TYPE_HYBRID = 3
+    GOOGLE_TYPE_SATELLITE = 4
+
+    GOOGLE_TYPE_CHOICES = (
+        (GOOGLE_TYPE_DEFAULT, 'google default'),
+        (GOOGLE_TYPE_PHYSICAL, 'google physical'),
+        (GOOGLE_TYPE_HYBRID, 'google hybrid'),
+        (GOOGLE_TYPE_SATELLITE, 'google satellite'),
+        )
+
+    name = models.CharField(max_length=20)
+    index = models.IntegerField(default=100)
+    default = models.BooleanField(default=False)
+    active = models.BooleanField(default=True)
+
+    layer_type = models.IntegerField(choices=LAYER_TYPE_CHOICES)
+    google_type = models.IntegerField(
+        choices=GOOGLE_TYPE_CHOICES,
+        null=True, blank=True,
+        help_text='Choose map type in case of GOOGLE maps.')
+    layer_url = models.CharField(
+        max_length=200,
+        null=True, blank=True,
+        help_text='Tile url for use with OSM or WMS',
+        default='http://tile.openstreetmap.nl/tiles/${z}/${x}/${y}.png')
+    layer_names = models.TextField(
+        null=True, blank=True,
+        help_text='Fill in layer names in case of WMS')
+
+    class Meta:
+        ordering = ('index', )
+
+    def __unicode__(self):
+        return '%s' % self.name
+
+
+class Setting(models.Model):
+    """
+    Global settings.
+
+    Available keys with default values:
+    projection 'EPSG:900913'
+    display_projection 'EPSG:4326'
+    googlemaps_api_key
+    """
+    key = models.CharField(max_length=20, unique=True)
+    value = models.CharField(max_length=200)
+
+    def __unicode__(self):
+        return '%s = %s' % (self.key, self.value)
+
+    @classmethod
+    def get(cls, key, default=None):
+        """
+        Return value from given key.
+
+        If the key does not exist, return None
+        """
+        try:
+            setting = cls.objects.get(key=key)
+            return setting.value
+        except cls.DoesNotExist:
+            logger.warn('Setting "%s" does not exist, taking default '
+                        'value "%s"' % (key, default))
+            return default
+
+    @classmethod
+    def get_dict(cls, key, default=None):
+        """
+        Return {key: value} for given key
+        """
+        return {key: cls.get(key, default)}
