@@ -10,7 +10,18 @@ from lizard_map.adapter import Graph
 from lizard_map.adapter import parse_identifier_json
 from lizard_map.adapter import workspace_item_image_url
 from lizard_map.animation import AnimationSettings
+from lizard_map.daterange import PERIOD_DAY
+from lizard_map.daterange import PERIOD_YEAR
+from lizard_map.daterange import PERIOD_OTHER
+from lizard_map.daterange import PERIOD_DAYS
+from lizard_map.daterange import SESSION_DT_START
+from lizard_map.daterange import SESSION_DT_END
 from lizard_map.daterange import current_start_end_dates
+from lizard_map.daterange import current_period
+from lizard_map.daterange import default_start
+from lizard_map.daterange import default_end
+from lizard_map.daterange import deltatime_range
+from lizard_map.daterange import set_date_range
 from lizard_map.dateperiods import ALL
 from lizard_map.dateperiods import YEAR
 from lizard_map.dateperiods import QUARTER
@@ -35,7 +46,6 @@ from lizard_map.utility import float_to_string
 from lizard_map.utility import short_string
 from lizard_map.workspace import WorkspaceItemAdapter
 from lizard_map.workspace import WorkspaceManager
-from lizard_map.templatetags import map as map_functions
 from lizard_map.templatetags import workspaces
 import lizard_map.admin
 import lizard_map.coordinates
@@ -82,6 +92,7 @@ class WorkspaceManagerTest(TestCase):
 
 
 class ViewsTest(TestCase):
+    fixtures = ('lizard_map', )
 
     class MockRequest(object):
         session = {}
@@ -140,7 +151,8 @@ class ViewsTest(TestCase):
         """
         client = Client()
         url = reverse('lizard_map.map_location_save')
-        response = client.post(url, {'x': 100, 'y': 150, 'zoom': 10})
+        response = client.post(url, {'left': 100, 'top': 100,
+                                     'right': 150, 'bottom': 150})
         self.assertEqual(response.status_code, 200)
 
     def test_map_location_load_default(self):
@@ -151,10 +163,10 @@ class ViewsTest(TestCase):
         self.assertEqual(response_load.status_code, 200)
         result = json.loads(response_load.content)
         self.assertEqual(
-            result,
-            {'x': '550000',
-             'y': '6850000',
-             'zoom': '10'})
+            result['extent'],
+            {'top': '6964942', 'right': '1254790',
+             'left': '-14675', 'bottom': '6668977'}
+            )
 
 
 class WorkspaceTest(TestCase):
@@ -364,93 +376,247 @@ class TestCollages(TestCase):
         self.assertTrue(workspace.collages.all())
 
 
-class TestAnimationSettings(TestCase):
-    """Tests for animation.py."""
+class TestDateRange(TestCase):
+    """Test daterange.py"""
 
     def setUp(self):
-
         class Mock(dict):
             pass
 
         self.request = Mock()
         self.request.session = Mock()
+        self.today = datetime.datetime(2011, 4, 21)
+        self.almost_one_day = datetime.timedelta(
+            hours=23, minutes=59, seconds=59)
 
-    def _fill_date_range(self):
-        """Helper method: fill session with date range info."""
+    def _test_set_date_range(self, request):
+
+        set_date_range(self.request, now=self.today)
+
+        # Get current period, dt_start, dt_end
+        period = current_period(self.request)
+        dt_start, dt_end = current_start_end_dates(
+            self.request, today=self.today)
+
+        return period, dt_start, dt_end
+
+    def test_current_start_end_dates(self):
+        dt_start, dt_end = current_start_end_dates(
+            self.request, today=self.today)
+        dt_start_expected = self.today + PERIOD_DAYS[PERIOD_DAY][0]
+        dt_end_expected = self.today + PERIOD_DAYS[PERIOD_DAY][1]
+
+        self.assertEquals(dt_start, dt_start_expected)
+        self.assertEquals(dt_end, dt_end_expected)
+
+    def test_current_period(self):
+        """Test default period."""
+        period = current_period(self.request)
+        self.assertEquals(period, PERIOD_DAY)
+
+    def test_set_date_range(self):
+        """Set date range to period_day, then retrieve it back"""
+        # Fake Post
+        self.request.method = 'POST'
+        self.request.POST = {
+            'period': str(PERIOD_DAY)}
+        self.request.META = {}
+
+        period, dt_start, dt_end = self._test_set_date_range(self.request)
+
+        self.assertEquals(period, PERIOD_DAY)
+        self.assertEquals(dt_start, self.today + PERIOD_DAYS[PERIOD_DAY][0])
+        self.assertEquals(dt_end, self.today + PERIOD_DAYS[PERIOD_DAY][1])
+
+    def test_set_date_range2(self):
+        """Set custom date range, then retrieve it back"""
+        timedelta_start = datetime.timedelta(days=-20)
+        timedelta_end = datetime.timedelta(days=15)
+
+        # Fake Post
+        self.request.method = 'POST'
+        self.request.POST = {
+            'period': str(PERIOD_OTHER),
+            'dt_start': self.today + timedelta_start,
+            'dt_end': self.today + timedelta_end}
+        self.request.META = {}
+
+        period, dt_start, dt_end = self._test_set_date_range(self.request)
+
+        self.assertEquals(period, PERIOD_OTHER)
+        self.assertEquals(dt_start, self.today + timedelta_start)
+        self.assertEquals(
+            dt_end, self.today + timedelta_end + self.almost_one_day)
+
+    def test_set_date_range3(self):
+        """Set start date after end date: result must have dt_start<dt_end"""
+        timedelta_start = datetime.timedelta(days=20)
+        timedelta_end = datetime.timedelta(days=-15)
+
+        # Fake Post
+        self.request.method = 'POST'
+        self.request.POST = {
+            'period': str(PERIOD_OTHER),
+            'dt_start': self.today + timedelta_start,
+            'dt_end': self.today + timedelta_end}
+        self.request.META = {}
+
+        period, dt_start, dt_end = self._test_set_date_range(self.request)
+
+        self.assertEquals(period, PERIOD_OTHER)
+        self.assertTrue(dt_start < dt_end)
+
+    def do_deltatime(
+        self, period_expected,
+        timedelta_start_expected, timedelta_end_expected):
+        """Easy testing deltatime_range."""
+
+        daterange = {'dt_start': self.today + timedelta_start_expected,
+                     'dt_end': self.today + timedelta_end_expected,
+                     'period': period_expected}
+        period, timedelta_start, timedelta_end = deltatime_range(
+            daterange, now=self.today)
+
+        # Test on day accuracy, because "almost_one_day" is added to end.
+        self.assertEquals(period, period_expected)
+        self.assertEquals(timedelta_start.days, timedelta_start_expected.days)
+        self.assertEquals(timedelta_end.days, timedelta_end_expected.days)
+
+    def test_deltatime_range(self):
+        """Deltatime_range"""
+        timedelta_start_expected = datetime.timedelta(-1000)
+        timedelta_end_expected = datetime.timedelta(20)
+        period_expected = PERIOD_OTHER
+        self.do_deltatime(
+            period_expected,
+            timedelta_start_expected, timedelta_end_expected)
+
+    def test_deltatime_range2(self):
+        """Deltatime_range"""
+        timedelta_start_expected = datetime.timedelta(-1)
+        timedelta_end_expected = datetime.timedelta(0)
+        period_expected = PERIOD_DAY
+        self.do_deltatime(
+            period_expected,
+            timedelta_start_expected, timedelta_end_expected)
+
+    def test_deltatime_range3(self):
+        """Deltatime_range"""
+        timedelta_start_expected = datetime.timedelta(-365)
+        timedelta_end_expected = datetime.timedelta(0)
+        period_expected = PERIOD_YEAR
+        self.do_deltatime(
+            period_expected,
+            timedelta_start_expected, timedelta_end_expected)
+
+
+class TestAnimationSettings(TestCase):
+    """Tests for animation.py."""
+
+    def _fill_date_range(self, today):
+        """Helper method: fill session with date range info.
+
+        date_start is 730119 days from day_one
+        date_end is 731215 days from day_one
+        """
         twothousand = datetime.datetime(year=2000, month=1, day=1)
         twothousandthree = datetime.datetime(year=2003, month=1, day=1)
-        self.request.session['date_start'] = twothousand
-        self.request.session['date_end'] = twothousandthree
+        self.request.session[SESSION_DT_START] = twothousand - today
+        self.request.session[SESSION_DT_END] = twothousandthree - today
+        day_one = datetime.datetime(1979, 5, 25)
+        self.date_start_days = (twothousand - day_one).days
+        self.date_end_days = (twothousandthree - day_one).days
 
-    def test_date_range_helper(self):
+    def _date_range_helper(self, today):
         """Make sure _fill_date_range() works."""
-        self._fill_date_range()
-        start, end = current_start_end_dates(self.request)
+        self._fill_date_range(today=today)
+        start, end = current_start_end_dates(self.request, today=today)
         self.assertEquals(start.year, 2000)
         self.assertEquals(end.year, 2003)
 
+    def setUp(self):
+        class Mock(dict):
+            pass
+
+        self.request = Mock()
+        self.request.session = Mock()
+        self.today = datetime.datetime(2011, 4, 21)
+        self._date_range_helper(today=self.today)  # Set the request datetimes
+
     def test_smoke(self):
-        animation_settings = AnimationSettings(request=self.request)
+        animation_settings = AnimationSettings(
+            request=self.request, today=self.today)
         self.assertTrue(animation_settings)  # It exists.
 
     def test_session_initialisation(self):
         self.assertFalse('animation_settings' in self.request.session)
-        AnimationSettings(self.request)
+        AnimationSettings(self.request, today=self.today)
         self.assertTrue('animation_settings' in self.request.session)
 
     def test_slider_position(self):
         """Are the getters/setters working?"""
-        animation_settings = AnimationSettings(self.request)
-        animation_settings.slider_position = 42
+        animation_settings = AnimationSettings(
+            self.request, today=self.today)
+        animation_settings.slider_position = self.date_start_days + 100
+        print current_start_end_dates(self.request)
         self.assertTrue(self.request.session.modified)
-        self.assertEquals(animation_settings.slider_position, 42)
-        self.assertEquals(self.request.session[
-                'animation_settings']['slider_position'], 42)
+        self.assertEquals(
+            animation_settings.slider_position, self.date_start_days + 100)
+        self.assertEquals(
+            self.request.session['animation_settings']['slider_position'],
+            self.date_start_days + 100)
 
     def test_initial_slider_position(self):
-        """Slider position should be 0 if not initialised.
+        """Slider position should be [max] if not initialised.
         In any case, it should not return a keyerror."""
-        animation_settings = AnimationSettings(self.request)
-        self.assertEquals(animation_settings.slider_position, 0)
+        animation_settings = AnimationSettings(
+            self.request, today=self.today)
+        self.assertEquals(animation_settings.slider_position,
+                          self.date_end_days)
 
     def test_initial_info_gathering(self):
         """Do we return the correct date range and position?"""
-        self._fill_date_range()
-        animation_settings = AnimationSettings(self.request)
+        self._fill_date_range(self.today)
+        animation_settings = AnimationSettings(
+            self.request, today=self.today)
         result = animation_settings.info()
-        self.assertEquals(result['min'], 0)
-        self.assertEquals(result['max'], 1096)
+        self.assertEquals(result['min'], self.date_start_days)
+        self.assertEquals(result['max'], self.date_end_days)
         self.assertEquals(result['step'], 1)
-        self.assertEquals(result['value'], 0)
-        self.assertEquals(result['selected_date'].year, 2000)
+        self.assertEquals(result['value'], self.date_end_days)
+        self.assertEquals(result['selected_date'].year, 2003)
 
     def test_info_gathering(self):
         """Do we return the correct date range and position?"""
-        self._fill_date_range()
-        animation_settings = AnimationSettings(self.request)
-        animation_settings.slider_position = 400
+        self._fill_date_range(self.today)
+        animation_settings = AnimationSettings(
+            self.request, today=self.today)
+        animation_settings.slider_position = self.date_start_days + 375
         result = animation_settings.info()
-        self.assertEquals(result['min'], 0)
-        self.assertEquals(result['max'], 1096)
+        self.assertEquals(result['min'], self.date_start_days)
+        self.assertEquals(result['max'], self.date_end_days)
         self.assertEquals(result['step'], 1)
-        self.assertEquals(result['value'], 400)
+        self.assertEquals(result['value'], self.date_start_days + 375)
         self.assertEquals(result['selected_date'].year, 2001)
 
     def test_impossible_negative_corner_case(self):
         """Negative dates."""
-        self._fill_date_range()
-        animation_settings = AnimationSettings(self.request)
+        self._fill_date_range(self.today)
+        animation_settings = AnimationSettings(
+            self.request, today=self.today)
         animation_settings.slider_position = -400
         result = animation_settings.info()
-        self.assertEquals(result['value'], 0)
+        self.assertEquals(result['value'], self.date_start_days)
 
     def test_impossible_beyond_max_corner_case(self):
         """Value beyond the max possible."""
-        self._fill_date_range()
-        animation_settings = AnimationSettings(self.request)
-        animation_settings.slider_position = 4000
+        self._fill_date_range(self.today)
+        animation_settings = AnimationSettings(
+            self.request, today=self.today)
+        animation_settings.slider_position = 1000000
         result = animation_settings.info()
-        self.assertEquals(result['value'], 1096)  # Max available.
+        # Max available.
+        self.assertEquals(result['value'], self.date_end_days)
 
 
 class UtilityTest(TestCase):
@@ -694,60 +860,60 @@ class WorkspaceItemAdapterTest(TestCase):
 class DatePeriodsTest(TestCase):
 
     def test_calc_aggregation_periods_all(self):
-        start_date = datetime.date(1979, 5, 25)
-        end_date = datetime.date(1980, 4, 15)
+        start_date = datetime.datetime(1979, 5, 25)
+        end_date = datetime.datetime(1980, 4, 15)
         periods = calc_aggregation_periods(start_date, end_date, ALL)
         self.assertEqual(periods[0][0], start_date)
         self.assertEqual(periods[0][1], end_date)
 
     def test_calc_aggregation_periods_year(self):
-        start_date = datetime.date(1979, 5, 25)
-        end_date = datetime.date(1980, 4, 15)
+        start_date = datetime.datetime(1979, 5, 25)
+        end_date = datetime.datetime(1980, 4, 15)
         periods = calc_aggregation_periods(start_date, end_date, YEAR)
         self.assertEqual(periods[0][0], start_date)
-        self.assertEqual(periods[0][1], datetime.date(1980, 1, 1))
-        self.assertEqual(periods[1][0], datetime.date(1980, 1, 1))
+        self.assertEqual(periods[0][1], datetime.datetime(1980, 1, 1))
+        self.assertEqual(periods[1][0], datetime.datetime(1980, 1, 1))
         self.assertEqual(periods[1][1], end_date)
 
     def test_calc_aggregation_periods_quarter(self):
-        start_date = datetime.date(1979, 5, 25)
-        end_date = datetime.date(1980, 4, 15)
+        start_date = datetime.datetime(1979, 5, 25)
+        end_date = datetime.datetime(1980, 4, 15)
         periods = calc_aggregation_periods(start_date, end_date, QUARTER)
         self.assertEqual(periods[0][0], start_date)
-        self.assertEqual(periods[0][1], datetime.date(1979, 7, 1))
-        self.assertEqual(periods[-1][0], datetime.date(1980, 4, 1))
+        self.assertEqual(periods[0][1], datetime.datetime(1979, 7, 1))
+        self.assertEqual(periods[-1][0], datetime.datetime(1980, 4, 1))
         self.assertEqual(periods[-1][1], end_date)
 
     def test_calc_aggregation_periods_month(self):
-        start_date = datetime.date(1979, 5, 25)
-        end_date = datetime.date(1980, 4, 15)
+        start_date = datetime.datetime(1979, 5, 25)
+        end_date = datetime.datetime(1980, 4, 15)
         periods = calc_aggregation_periods(start_date, end_date, MONTH)
         self.assertEqual(periods[0][0], start_date)
-        self.assertEqual(periods[0][1], datetime.date(1979, 6, 1))
-        self.assertEqual(periods[-1][0], datetime.date(1980, 4, 1))
+        self.assertEqual(periods[0][1], datetime.datetime(1979, 6, 1))
+        self.assertEqual(periods[-1][0], datetime.datetime(1980, 4, 1))
         self.assertEqual(periods[-1][1], end_date)
 
     def test_calc_aggregation_periods_week(self):
-        start_date = datetime.date(1979, 5, 25)  # It's a friday.
-        end_date = datetime.date(1979, 7, 15)  # It's a sunday.
+        start_date = datetime.datetime(1979, 5, 25)  # It's a friday.
+        end_date = datetime.datetime(1979, 7, 15)  # It's a sunday.
         periods = calc_aggregation_periods(start_date, end_date, WEEK)
         self.assertEqual(periods[0][0], start_date)
-        self.assertEqual(periods[0][1], datetime.date(1979, 5, 28))
-        self.assertEqual(periods[-1][0], datetime.date(1979, 7, 9))
+        self.assertEqual(periods[0][1], datetime.datetime(1979, 5, 28))
+        self.assertEqual(periods[-1][0], datetime.datetime(1979, 7, 9))
         self.assertEqual(periods[-1][1], end_date)
 
     def test_calc_aggregation_periods_day(self):
-        start_date = datetime.date(1979, 5, 25)
-        end_date = datetime.date(1979, 7, 15)
+        start_date = datetime.datetime(1979, 5, 25)
+        end_date = datetime.datetime(1979, 7, 15)
         periods = calc_aggregation_periods(start_date, end_date, DAY)
         self.assertEqual(periods[0][0], start_date)
-        self.assertEqual(periods[0][1], datetime.date(1979, 5, 26))
-        self.assertEqual(periods[-1][0], datetime.date(1979, 7, 14))
+        self.assertEqual(periods[0][1], datetime.datetime(1979, 5, 26))
+        self.assertEqual(periods[-1][0], datetime.datetime(1979, 7, 14))
         self.assertEqual(periods[-1][1], end_date)
 
     def test_fancy_period(self):
-        start_date = datetime.date(1979, 5, 25)
-        end_date = datetime.date(1979, 7, 15)
+        start_date = datetime.datetime(1979, 5, 25)
+        end_date = datetime.datetime(1979, 7, 15)
         self.assertTrue(fancy_period(start_date, end_date, ALL))
         self.assertTrue(fancy_period(start_date, end_date, YEAR))
         self.assertTrue(fancy_period(start_date, end_date, QUARTER))
@@ -772,18 +938,6 @@ class TestTemplateTags(TestCase):
         for value, expected in in_out.items():
             self.assertEquals(workspaces.float_or_exp(value), expected)
 
-    def test_map_variables(self):
-        """Just see if map_variables crashes"""
-        parser = None
-        token = None
-        map_functions.map_variables(parser, token)
-
-    def test_detect_browser(self):
-        """Just see if detect_browser crashes"""
-        parser = None
-        token = None
-        map_functions.detect_browser(parser, token)
-
 
 class ModelLegendTest(TestCase):
 
@@ -942,6 +1096,7 @@ class TestOperations(TestCase):
 
 
 class CoordinatesTest(TestCase):
+    fixtures = ('lizard_map', )
 
     def test_detect_prj1(self):
         prj = None
@@ -975,3 +1130,49 @@ class CoordinatesTest(TestCase):
         self.assertEquals(
             lizard_map.coordinates.detect_prj(prj),
             lizard_map.coordinates.WGS84)
+
+    def test_map_settings(self):
+        """
+        Test MapSettings class. Nothing in settings.py, so revert to
+        default google projection and srid.
+        """
+        map_settings = lizard_map.coordinates.MapSettings()
+
+        self.assertTrue(map_settings.mapnik_projection(),
+                        lizard_map.coordinates.GOOGLE)
+        self.assertTrue(map_settings.srid, 900913)
+
+    def test_map_settings2(self):
+        """
+        Test custom settings for MapSettings.
+        """
+
+        MAP_SETTINGS = {
+            'base_layer_type': 'WMS',  # OSM or WMS
+            'projection': 'EPSG:28992',  # EPSG:900913, EPSG:28992
+            'display_projection': 'EPSG:28992',  # EPSG:900913/28992/4326
+            'startlocation_x': '144000',
+            'startlocation_y': '486000',
+            'startlocation_zoom': '4',
+            'base_layer_wms': (
+                'http://kaart.almere.nl/wmsconnector/com.esri.wms.Esrimap?'
+                'SERVICENAME=AIKWMS&'),
+            'base_layer_wms_layers': (
+                'mrsid2008'),
+            }
+        map_settings = lizard_map.coordinates.MapSettings(MAP_SETTINGS)
+        self.assertTrue(map_settings.mapnik_projection(),
+                        lizard_map.coordinates.RD)
+        self.assertTrue(map_settings.srid, 28992)
+
+    def test_srs_to_google(self):
+        x, y = lizard_map.coordinates.srs_to_google(
+            'EPSG:28992', 35219, 467574)
+        self.assertTrue(abs(x - 404721) < 1)
+        self.assertTrue(abs(y - 6833334) < 1)
+
+    def test_google_to_srs(self):
+        x, y = lizard_map.coordinates.google_to_srs(
+            579427, 6860742, 'EPSG:28992')
+        self.assertTrue(abs(x - 142586) < 1)
+        self.assertTrue(abs(y - 482911) < 1)
