@@ -1,6 +1,5 @@
 import StringIO
 
-from django.conf import settings
 from django.db.models import Max
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404
@@ -15,8 +14,8 @@ import mapnik
 
 from lizard_map import coordinates
 from lizard_map.adapter import parse_identifier_json
-from lizard_map.animation import AnimationSettings
 from lizard_map.animation import slider_layout_extra
+from lizard_map.coordinates import MapSettings
 from lizard_map.daterange import current_start_end_dates
 from lizard_map.daterange import DateRangeForm
 from lizard_map.models import Workspace
@@ -38,7 +37,7 @@ logger = logging.getLogger(__name__)
 
 
 def homepage(request,
-             template='lizard_map/example_homepage.html',
+             template='lizard_map/app_screen.html',
              crumbs_prepend=None,
              application_screen_slug=None):
     """Default apps screen, make your own template.
@@ -46,13 +45,6 @@ def homepage(request,
     Optionally, if application_screen_slug is None, try to fetch GET
     parameter 'screen' from url.
     """
-    date_range_form = DateRangeForm(
-        current_start_end_dates(request, for_form=True))
-
-    workspace_manager = WorkspaceManager(request)
-    workspaces = workspace_manager.load_or_create()
-    date_range_form = DateRangeForm(
-        current_start_end_dates(request, for_form=True))
 
     if crumbs_prepend is not None:
         crumbs = list(crumbs_prepend)
@@ -64,9 +56,7 @@ def homepage(request,
 
     return render_to_response(
         template,
-        {'date_range_form': date_range_form,
-         'workspaces': workspaces,
-         'javascript_hover_handler': 'popup_hover_handler',
+        {'javascript_hover_handler': 'popup_hover_handler',
          'javascript_click_handler': 'popup_click_handler',
          'crumbs': crumbs,
          'application_screen_slug': application_screen_slug},
@@ -84,24 +74,12 @@ def workspace(request,
     """
     workspace = get_object_or_404(Workspace, pk=workspace_id)
 
-    # Calculate whether we want animation.
-    # TODO: make custom RequestContext where this code is included
-    animation_slider = False
-    for workspace_item in workspace.workspace_items.filter(
-        visible=True):
-        if workspace_item.adapter.is_animatable:
-            animation_slider = AnimationSettings(request).info()
-            break
-
-    date_range_form = DateRangeForm(
-        current_start_end_dates(request, for_form=True))
     return render_to_response(
         template,
         {'workspaces': {'user': [workspace]},
          'javascript_click_handler': javascript_click_handler,
          'javascript_hover_handler': javascript_hover_handler,
-         'animation_slider': animation_slider,
-         'date_range_form': date_range_form},
+         },
         context_instance=RequestContext(request))
 
 
@@ -197,10 +175,18 @@ def workspace_item_edit(request, workspace_item_id=None, visible=None):
 @never_cache
 def workspace_item_extent(request, workspace_item_id=None):
     """Returns extent for the workspace in json.
+
+    Transform to correct client-side projection, then return coordinates.
     """
     workspace_item_id = request.GET['workspace_item_id']
     workspace_item = get_object_or_404(WorkspaceItem, pk=workspace_item_id)
-    return HttpResponse(json.dumps(workspace_item.adapter.extent()))
+    extent = workspace_item.adapter.extent()
+
+    map_settings = MapSettings()
+    extent_converted = map_settings.convert_google_extent_map_srs(
+        extent['east'], extent['north'],
+        extent['west'], extent['south'])
+    return HttpResponse(json.dumps(extent_converted))
 
 
 @never_cache
@@ -364,6 +350,9 @@ def session_workspace_extent(request, workspace_category='user'):
 def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
     """Return html with info on list of 'found' objects.
 
+    Optionally give pagenumber (starts at 0). If omitted, just join
+    everything.
+
     found: list of dictionaries {'distance': ..., 'timeserie': ...,
     'workspace_item': ..., 'identifier': ...}.
 
@@ -379,17 +368,20 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
               }
     """
 
-    result_html = ''
+    html = {}
     x_found = None
     y_found = None
 
     # Regroup found list of objects into workspace_items.
     display_groups = {}
+    display_group_order = []
     for display_object in found:
         workspace_item = display_object['workspace_item']
         if workspace_item.id not in display_groups:
             display_groups[workspace_item.id] = []
         display_groups[workspace_item.id].append(display_object)
+        if workspace_item.id not in display_group_order:
+            display_group_order.append(workspace_item.id)
 
     if len(display_groups) > 1:
         big_popup = True
@@ -433,7 +425,9 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
 
         if 'google_coords' in display_object:
             x_found, y_found = display_object['google_coords']
-        result_html += html_per_workspace_item
+        html[workspace_item.id] = html_per_workspace_item
+
+    result_html = [html[index] for index in display_group_order][:3]
 
     if popup_id is None:
         popup_id = 'popup-id'
@@ -449,9 +443,11 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
 def popup_collage_json(collage, popup_id, request=None):
     """
     display collage by adding display_groups together
+
+    TODO: see popup_json
     """
 
-    result_html = ''
+    html = []
     snippet_groups = collage.snippet_groups.all()
     if len(snippet_groups) > 1:
         big_popup = True
@@ -467,7 +463,7 @@ def popup_collage_json(collage, popup_id, request=None):
             html_per_workspace_item = workspace_item.adapter.html(
                 snippet_group=snippet_group,
                 layout_options={'legend': True})
-            result_html += html_per_workspace_item
+            html.append(html_per_workspace_item)
 
             # Pick the location of the first snippet.
             if 'google_coords' in snippets[0].location:
@@ -476,7 +472,7 @@ def popup_collage_json(collage, popup_id, request=None):
     result = {'id': popup_id,
               'x': google_x,
               'y': google_y,
-              'html': result_html,
+              'html': html,
               'big': big_popup,
               }
     return HttpResponse(json.dumps(result))
@@ -490,8 +486,6 @@ def collage(request,
             editable=False,
             template='lizard_map/collage.html'):
     """Render page with one collage"""
-    date_range_form = DateRangeForm(
-        current_start_end_dates(request, for_form=True))
     show_table = request.GET.get('show_table', False)
 
     collage = get_object_or_404(WorkspaceCollage, pk=collage_id)
@@ -500,7 +494,6 @@ def collage(request,
         template,
         {'collage': collage,
          'editable': editable,
-         'date_range_form': date_range_form,
          'request': request,
          'show_table': show_table},
         context_instance=RequestContext(request))
@@ -778,6 +771,12 @@ def search_name(request):
     srs = request.GET.get('srs')
     google_x, google_y = coordinates.srs_to_google(srs, x, y)
 
+    # Add a workspace, if it's not already in your own workspaces.
+    user_workspace_id = request.GET.get('user_workspace_id', None)
+    if user_workspace_id is not None:
+        workspace_manager.add_other(user_workspace_id)
+        workspace_collections = workspace_manager.workspaces
+
     found = []
     for workspace_collection in workspace_collections.values():
         for workspace in workspace_collection:
@@ -824,6 +823,12 @@ def search_coordinates(request):
     srs = request.GET.get('srs')
     google_x, google_y = coordinates.srs_to_google(srs, x, y)
 
+    # Add a workspace, if it's not already in your own workspaces.
+    user_workspace_id = request.GET.get('user_workspace_id', None)
+    if user_workspace_id is not None:
+        workspace_manager.add_other(user_workspace_id)
+        workspace_collections = workspace_manager.workspaces
+
     found = []
     for workspace_collection in workspace_collections.values():
         for workspace in workspace_collection:
@@ -932,32 +937,26 @@ contains a dictionary with fields x, y and zoom.
 
 def map_location_save(request):
     """
-    Saves given coordinates as strings (POST x, y, zoom) to session.
+    Save extent as strings (POST top, left, right, bottom) to session.
     """
-    x = request.POST['x']
-    y = request.POST['y']
-    zoom = request.POST['zoom']
-    request.session[MAP_LOCATION] = {'x': x, 'y': y, 'zoom': zoom}
+    top = request.POST['top']
+    left = request.POST['left']
+    right = request.POST['right']
+    bottom = request.POST['bottom']
+    request.session[MAP_LOCATION] = {
+        'top': top,
+        'left': left,
+        'right': right,
+        'bottom': bottom}
     return HttpResponse("")
 
 
 def map_location_load_default(request):
     """
-    Returns stored coordinates in a json dict, or empty dict if
-    nothing was saved.
+    Return start_extent
     """
-    try:
-        map_settings = settings.MAP_SETTINGS
-        x = map_settings['startlocation_x']
-        y = map_settings['startlocation_y']
-        zoom = map_settings['startlocation_zoom']
-    except AttributeError:
-        logger.warn(
-            'Could not find MAP_SETTINGS in '
-            'django settings or MAP_SETTINGS '
-            'not properly configured, using default coordinates.')
-        x, y, zoom = '550000', '6850000', '10'
-
-    map_location = {'x': x, 'y': y, 'zoom': zoom}
+    map_settings = MapSettings()
+    extent = map_settings.map_settings['start_extent']
+    map_location = {'extent': extent}
 
     return HttpResponse(json.dumps(map_location))
