@@ -9,6 +9,7 @@ from django.utils.translation import ugettext as _
 
 from lizard_map.models import DEFAULT_WORKSPACES
 from lizard_map.models import ICON_ORIGINALS
+from lizard_map.models import OTHER_WORKSPACES
 from lizard_map.models import TEMP_WORKSPACES
 from lizard_map.models import USER_WORKSPACES
 from lizard_map.models import Color
@@ -135,6 +136,28 @@ class WorkspaceManager:
             self.save_workspaces()
         return self.workspaces
 
+    def _in_collection(self, workspace):
+        """Check if given workspace is already in current collection.
+        """
+        for group, workspaces in self.workspaces.items():
+            if workspace in workspaces:
+                return True
+        return False
+
+    def add_other(self, workspace_id):
+        """
+        Add a workspace in "other",
+
+        if the workspace is not already in your collection.
+
+        Sometimes you want to add workspaces from others
+        """
+        workspace = Workspace.objects.get(pk=workspace_id)
+        if not self._in_collection(workspace):
+            if OTHER_WORKSPACES not in self.workspaces:
+                self.workspaces[OTHER_WORKSPACES] = []
+            self.workspaces[OTHER_WORKSPACES].append(workspace)
+
 
 class WorkspaceItemAdapter(object):
     """Base class for workspace_item adapters.
@@ -383,7 +406,8 @@ class WorkspaceItemAdapter(object):
         return 'html output for this adapter is not implemented'
 
     def html_default(self, snippet_group=None, identifiers=None,
-                     layout_options=None):
+                     layout_options=None, template=None,
+                     extra_render_kwargs=None):
         """
         Returns html representation of given snippet_group OR
         identifiers (snippet_group has priority). If a snippet_group
@@ -441,18 +465,25 @@ class WorkspaceItemAdapter(object):
         display_group = [self.location(**identifier) for identifier in
                          identifiers]
 
-        return render_to_string(
-            'lizard_map/popup.html',
-            {
-                'title': title,
-                'display_group': display_group,
-                'img_url': img_url,
-                'symbol_url': self.symbol_url(),
-                'add_snippet': add_snippet,
-                'editing': editing,
-                'snippet_group': snippet_group,
+        if template is None:
+            template = 'lizard_map/popup.html'
+        render_kwargs = {
+            'title': title,
+            'display_group': display_group,
+            'img_url': img_url,
+            'symbol_url': self.symbol_url(),
+            'add_snippet': add_snippet,
+            'editing': editing,
+            'snippet_group': snippet_group,
                 'colors': COLORS_DEFAULT,
-                },
+            }
+
+        if extra_render_kwargs is not None:
+            render_kwargs.update(extra_render_kwargs)
+
+        return render_to_string(
+            template,
+            render_kwargs
             )
 
     def legend(self, updates=None):
